/*
   $Header: /n/pho/da/krste/realsys/phipac/mul_mfvf_vf/RCS/mul_mfvf_vf_test.c,v 1.3 1996/10/30 01:53:46 krste Exp $

   Test mul_mfvf_vf routine.
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec.h"
#include "rtst.h"

extern float
php_mul_mfvf_vf(size_t rows, size_t cols,
                const float* mat, const float* invec, const float *outvec);

#undef mul_mfvf_vf

#define mul_mfvf_vf(r,c,m,v,res) php_mul_mfvf_vf(r,c,m,v,res)

/* Performs one test given input values and expected output. */

void
one_test(size_t rows, size_t cols,
         const float* mat, const float* invec, const float* resvec)
{
    /* Create padded copies of inputs. */
    const size_t mat_len = rows * cols;
    float* c_mat = rtst_padvec_clone_vf(mat_len, mat);
    float* c_invec = rtst_padvec_clone_vf(cols, invec);
    float* c_resvec = rtst_padvec_new_vf(rows);

    /* Run routine. */
    mul_mfvf_vf(rows, cols, c_mat, c_invec, c_resvec);

    /* Check result. */
    rtst_checkeq_vfvf(rows, c_resvec, resvec);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(mat_len, c_mat, mat);
    rtst_checkeq_vfvf(cols, c_invec, invec);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_mat);
    rtst_padvec_del_vf(c_invec);
    rtst_padvec_del_vf(c_resvec);
}

void
test_1x1(void)
{
    static const float m[] = { 12.0f };
    static const float v[] = {  5.0f };
    static const float r[] = { 60.0f };

    one_test(1, 1, m, v, r);
}

void
test_3x2(void)
{
    static const float m[] = {
        1.0f,  3.0f,
        -2.0f,  9.0f,
        3.0f, -3.0f
    };
    static const float v[] = { 7.0f, -5.0f };
    static const float r[] = { -8.0f, -59.0f, 36.0f };

    one_test(3, 2, m, v, r);
}

void
run_directed_tests(void)
{
    test_1x1();
    test_3x2();
}

void
run_random_tests(void)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    float *mat = malloc_vf(maxlen*maxlen);
    float *invec = malloc_vf(maxlen);
    float *outvec = malloc_vf(maxlen);

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t rows = (size_t) rtst_urand_i32i32_i32(1, maxlen);
        size_t cols = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        rtst_urandi_i32i32_vf(rows*cols, -7, +7, mat);
        rtst_urandi_i32i32_vf(cols, -7, +7, invec);

        nv_mul_mfvf_vf(rows, cols, mat, invec, outvec);

        rtst_log("rows=%d cols=%d\n", rows, cols);

        one_test(rows, cols, mat, invec, outvec);
    }

    free_vf(mat);
    free_vf(invec);
    free_vf(outvec);
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    rtst_start("Directed tests of mul_mfvf_vf.");
    run_directed_tests();
    rtst_passed();

    rtst_start("Random tests against naive code.");
    run_random_tests();
    rtst_passed();
    
    rtst_exit();

    return 0;
}
